import logging
import os

from notetool.secret import read_secret
from sqlalchemy import MetaData, Table, create_engine
from sqlalchemy.orm import declarative_base

uri = read_secret(cate1='notecoin', cate2='dataset', cate3='db_path')
uri = uri or f'sqlite:///{os.path.abspath(os.path.dirname(__file__))}/data/notecoin.db'
# engine = create_engine(uri, echo=True)
meta = MetaData()
engine = create_engine(uri)
Base = declarative_base()
logging.info(f'uri:{uri}')


class BaseTable:
    def __init__(self, table_name, *args, **kwargs):
        self.table_name = table_name
        self.table: Table
        meta.create_all(engine)

    def insert(self, values, keys=None, *args, **kwargs):
        meta.create_all(engine)
        cols = [col.name for col in self.table.columns]
        if isinstance(values, dict):
            values = dict([(k, v) for k, v in values.items() if k in cols])
        elif isinstance(values, list):
            if isinstance(values[0], dict):
                values = [dict(zip(keys, item)) for item in values]
            elif isinstance(values[0], list):
                values = [dict([(k, v) for k, v in item.items() if k in cols]) for item in values]
        ins = self.table.insert(values=values).prefix_with("OR REPLACE")
        engine.execute(ins)



from sqlalchemy import BIGINT, Column, Float, String, Table

from .base import BaseTable, meta


class KlineData(BaseTable):
    def __init__(self, table_name="kline_data_1min", *args, **kwargs):
        super(KlineData, self).__init__(table_name=table_name, *args, **kwargs)
        self.table = Table(self.table_name, meta,
                           Column('symbol', String(30), comment='symbol', primary_key=True),
                           Column('timestamp', BIGINT, comment='timestamp', primary_key=True),
                           Column('open', Float, comment='open', default=0),
                           Column('close', Float, comment='close', default=0),
                           Column('low', Float, comment='low', default=0),
                           Column('high', Float, comment='high', default=0),
                           Column('vol', Float, comment='vol', default=0))



import logging

import ccxt
from ccxt.base.exchange import Exchange

# from notecoin.base.database import KlineData
# data = KlineData()
# values = [
#     {'source': 'okex', 'symbol': 'a', 'timestamp': '34'},
#     {'source': 'okex', 'symbol': 'a', 'timestamp': '35'},
#     {'source': 'okex', 'symbol': 'a', 'timestamp': '36'},
#     {'source': 'okex', 'symbol': 'a', 'timestamp': '37'},
# ]
# data.insert(values)


class LoadDataKline:
    def __init__(self, exchange: Exchange, *args, **kwargs):
        self.exchange = exchange
        super(LoadDataKline, self).__init__(*args, **kwargs)

    def load_all(self):
        logging.info('Loading', self.exchange.id, 'markets...')
        self.exchange.load_markets()
        for sym in self.exchange.symbols:
            print(sym)
        logging.info('Loaded', self.exchange.id, 'markets.')

    def load(self, symbol, timeframe='1m', max_retries=3):

        limit = 100
        earliest_timestamp = self.exchange.milliseconds()
        timeframe_duration_in_seconds = self.exchange.parse_timeframe(timeframe)
        timeframe_duration_in_ms = timeframe_duration_in_seconds * 1000
        timedelta = limit * timeframe_duration_in_ms
        ohlcv_dictionary = {}
        ohlcv_list = []
        i = 0
        done = False
        while True:
            logging.info('===========================================================')
            logging.info('Iteration', i)
            fetch_since = earliest_timestamp - timedelta
            logging.info('Fetching', self.exchange.id, symbol, timeframe, 'candles from',
                         self.exchange.iso8601(fetch_since), 'to',
                         self.exchange.iso8601(earliest_timestamp))
            num_retries = 0
            try:
                num_retries += 1
                ohlcv = self.exchange.fetch_ohlcv(symbol, timeframe, fetch_since, limit)
                if len(ohlcv):
                    earliest_timestamp = ohlcv[0][0] - timeframe_duration_in_ms
                    print('Fetched', len(ohlcv), self.exchange.id, symbol, timeframe, 'candles from',
                          self.exchange.iso8601(ohlcv[0][0]), 'to', self.exchange.iso8601(ohlcv[-1][0]))
                else:
                    print('Fetched', len(ohlcv), self.exchange.id, symbol, timeframe, 'candles')
                    done = True
            except Exception as e:
                logging.error(e)
                if num_retries > max_retries:
                    raise
                else:
                    continue
            i += 1
            ohlcv_dictionary = self.exchange.extend(ohlcv_dictionary, self.exchange.indexBy(ohlcv, 0))
            ohlcv_list = self.exchange.sort_by(ohlcv_dictionary.values(), 0)
            if len(ohlcv_list):
                print(ohlcv_list)
                print('Stored', len(ohlcv_list), self.exchange.id, symbol, timeframe, 'candles from',
                      self.exchange.iso8601(ohlcv_list[0][0]), 'to', self.exchange.iso8601(ohlcv_list[-1][0]))
            if done:
                break
        return ohlcv_list


exchan = LoadDataKline(ccxt.okex())

symbol = 'BTC/USDT'

ohlcvs = exchan.load(symbol)
print('Done.')
